import copy
from unittest import TestCase
from unittest.mock import Mock, patch

import click
from box import Box

from montecarlodata.common.data import AWSArn, DcResourceProperties
from montecarlodata.common.resources import CloudResourceService
from montecarlodata.discovery.networking import NetworkDiscoveryService

SAMPLE_RESOURCE_ID = "spensa"
SAMPLE_DC_PROPS = DcResourceProperties(
    collector_arn=AWSArn("arn:aws:cloudformation:us-east-1:1234:stack/foo/bar"),
    collector_props=Box(
        {
            "stackArn": "arn:aws:cloudformation:us-east-1:1234:stack/foo/bar",
            "templateVersion": "42",
            "codeVersion": "2340",
            "lastUpdated": None,
            "outputs": {
                "public_route_table": "rtb-1234",
                "private_route_table": "rtb-456",
                "vpc_id": "vpc-1234",
                "public_ip": "127.0.0.1",
                "cloud_watch_log_vpc_endpoint": "vpce-1234",
                "template_version": "2340",
                "api_gateway_id": "foo",
                "private_s3_bucket_arn": "arn:aws:s3:::bucket",
                "security_group": "sg-1234",
            },
            "parameters": {
                "enable_gateway_access_logging": "False",
                "create_event_infra": "False",
                "existing_s3_vpce": "N/A",
                "worker_lambda_concurrent_executions": "42",
                "subnet_cid_rs": "10.30.0.0/24,10.30.1.0/24,10.30.2.0/24,10.30.3.0/24",
                "existing_vpc_id": "N/A",
                "create_cross_account_role": "True",
                "existing_subnet_ids": "N/A",
                "vpc_cidr": "10.30.0.0/16",
            },
        }
    ),
    collection_region="us-east-1",
    resources_region="us-east-1",
    collection_client=Mock(),
    resources_client=Mock(),
)
SAMPLE_RESOURCE_PROPS = Box(
    {
        "cluster_identifier": "test-cluster-pc",
        "cluster_namespace_arn": "arn:aws:redshift:us-east-1:1234:namespace:5678",
        "node_type": "dc2.large",
        "endpoint": {"address": "foo.com", "port": 5439},
        "vpc_security_groups": [
            {"vpc_security_group_id": "sg-0987", "status": "active"},
            {"vpc_security_group_id": "sg-6543", "status": "active"},
        ],
        "cluster_security_groups": [],
        "publicly_accessible": True,
        "vpc_id": "vpc-5678",
    }
)


class NetworkDiscoveryTest(TestCase):
    def setUp(self) -> None:
        self.maxDiff = None
        self._cloud_resource_service_mock = Mock(autospec=CloudResourceService)

        self._service = NetworkDiscoveryService(
            cloud_resource_service=self._cloud_resource_service_mock,
            command_name="test",
        )

    def tearDown(self) -> None:
        SAMPLE_DC_PROPS.collection_client.reset_mock()
        SAMPLE_DC_PROPS.resources_client.reset_mock()

    @patch.object(NetworkDiscoveryService, "rs_network_recommender")
    def test_recommend_network_dispatcher(self, rs_network_recommender_mock):
        resource_type, dc_id = "redshift", "42"
        collector_props = dc_resource_props = Box({"foo": "bar"})

        self._cloud_resource_service_mock.get_and_validate_active_collector.return_value = (
            collector_props
        )
        self._cloud_resource_service_mock.get_dc_resource_props.return_value = collector_props

        self._service.recommend_network_dispatcher(resource_type=resource_type, dc_id=dc_id)
        self._cloud_resource_service_mock.get_and_validate_active_collector.assert_called_once_with(
            dc_id=dc_id
        )
        self._cloud_resource_service_mock.get_dc_resource_props.assert_called_once_with(
            collector_props=collector_props
        )

        rs_network_recommender_mock.assert_called_once_with(dc_resource_props=dc_resource_props)

    @patch.object(NetworkDiscoveryService, "rs_network_recommender")
    @patch("montecarlodata.discovery.networking.complain_and_abort")
    def test_recommend_network_dispatcher_with_invalid_resource_type(
        self, abort_mock, rs_network_recommender_mock
    ):
        self._service.recommend_network_dispatcher(resource_type="foo")
        abort_mock.assert_called_once_with("Unsupported resource type - 'foo'.")
        rs_network_recommender_mock.assert_not_called()

    @patch.object(NetworkDiscoveryService, "_get_rs_sg")
    @patch.object(NetworkDiscoveryService, "_echo_recommendation")
    @patch.object(NetworkDiscoveryService, "_get_ip_filtering_verbiage")
    def test_rs_network_recommender_with_filtering(self, ip_filtering_mock, echo_mock, sg_mock):
        filtering_body, sg = ["foo", "bar", "qux"], ["sg-1234", "sg-5678"]
        SAMPLE_DC_PROPS.resources_client.describe_vpc.return_value = Box(cidr_block="127.0.0.1/32")

        SAMPLE_DC_PROPS.resources_client.describe_redshift_cluster.return_value = (
            SAMPLE_RESOURCE_PROPS
        )
        ip_filtering_mock.return_value = filtering_body
        sg_mock.return_value = sg, True
        self._service.rs_network_recommender(
            dc_resource_props=SAMPLE_DC_PROPS, resource_identifier=SAMPLE_RESOURCE_ID
        )

        ip_filtering_mock.assert_called_once_with(SAMPLE_DC_PROPS, sg)
        sg_mock.assert_called_once_with(SAMPLE_DC_PROPS)
        echo_mock.assert_called_once_with(
            filtering_body,
            header_col="The redshift cluster 'spensa' is publicly accessible.",
        )
        SAMPLE_DC_PROPS.resources_client.describe_vpc.assert_called_once_with(
            SAMPLE_RESOURCE_PROPS.vpc_id
        )
        SAMPLE_DC_PROPS.resources_client.describe_redshift_cluster.assert_called_once_with(
            cluster_id=SAMPLE_RESOURCE_ID
        )

    @patch.object(NetworkDiscoveryService, "_echo_recommendation")
    @patch("montecarlodata.discovery.networking.complain_and_abort")
    @patch.object(NetworkDiscoveryService, "_get_rs_sg")
    def test_rs_network_with_invalid_vpc_type(self, sg_mock, abort_mock, echo_mock):
        resource_props = SAMPLE_RESOURCE_PROPS
        resource_props.publicly_accessible = False

        SAMPLE_DC_PROPS.resources_client.describe_redshift_cluster.return_value = resource_props
        sg_mock.return_value = [], False
        abort_mock.side_effect = click.Abort()

        with self.assertRaises(click.exceptions.Abort):
            self._service.rs_network_recommender(
                dc_resource_props=SAMPLE_DC_PROPS,
                resource_identifier=SAMPLE_RESOURCE_ID,
            )
        SAMPLE_DC_PROPS.resources_client.describe_redshift_cluster.assert_called_once_with(
            cluster_id=SAMPLE_RESOURCE_ID
        )
        SAMPLE_DC_PROPS.resources_client.describe_vpc.assert_called_once_with(
            SAMPLE_RESOURCE_PROPS.vpc_id
        )

        abort_mock.assert_called_once_with(
            "Recommender does not support non-public clusters deployed outside a VPC."
        )
        echo_mock.assert_not_called()
        sg_mock.assert_called_once_with(SAMPLE_DC_PROPS)

    @patch.object(NetworkDiscoveryService, "_get_peering_intro_verbiage")
    @patch.object(NetworkDiscoveryService, "_get_rs_sg")
    @patch.object(NetworkDiscoveryService, "_echo_recommendation")
    @patch.object(NetworkDiscoveryService, "_get_overlap_verbiage")
    @patch("montecarlodata.discovery.networking.is_overlap")
    def test_rs_network_with_with_overlap(
        self, is_overlap_mock, overlap_verbiage_mock, echo_mock, sg_mock, intro_mock
    ):
        rs_cidr = "127.0.0.1/32"
        overlap_verbiage, intro_verbiage = ["foo", "bar"], ["qux", "baz"]
        resource_props = SAMPLE_RESOURCE_PROPS
        resource_props.publicly_accessible = False

        SAMPLE_DC_PROPS.resources_client.describe_redshift_cluster.return_value = resource_props
        sg_mock.return_value = [], True
        is_overlap_mock.return_value = True
        overlap_verbiage_mock.return_value = overlap_verbiage
        intro_mock.return_value = intro_verbiage
        SAMPLE_DC_PROPS.resources_client.describe_vpc.return_value = Box(cidr_block=rs_cidr)

        self._service.rs_network_recommender(
            dc_resource_props=SAMPLE_DC_PROPS, resource_identifier=SAMPLE_RESOURCE_ID
        )

        SAMPLE_DC_PROPS.resources_client.describe_redshift_cluster.assert_called_once_with(
            cluster_id=SAMPLE_RESOURCE_ID
        )
        SAMPLE_DC_PROPS.resources_client.describe_vpc.assert_called_once_with(
            SAMPLE_RESOURCE_PROPS.vpc_id
        )
        is_overlap_mock.assert_called_once_with(
            rs_cidr, SAMPLE_DC_PROPS.collector_props.parameters.vpc_cidr
        )
        overlap_verbiage_mock.assert_called_once_with(
            SAMPLE_DC_PROPS.collector_props.parameters.vpc_cidr, rs_cidr
        )
        sg_mock.assert_called_once_with(SAMPLE_DC_PROPS)
        intro_mock.assert_called_once_with(True)
        echo_mock.assert_called_once_with(
            [*intro_verbiage, overlap_verbiage],
            header_col="The redshift cluster 'spensa' is not publicly accessible.",
        )

    @patch.object(NetworkDiscoveryService, "_get_rs_routes")
    @patch.object(NetworkDiscoveryService, "_echo_recommendation")
    @patch.object(NetworkDiscoveryService, "_get_extra_verbiage")
    @patch.object(NetworkDiscoveryService, "_get_rs_accepter_verbiage")
    @patch.object(NetworkDiscoveryService, "_get_rs_requester_verbiage")
    @patch.object(NetworkDiscoveryService, "_get_rs_cross_account_verbiage")
    @patch.object(NetworkDiscoveryService, "_get_peering_intro_verbiage")
    @patch.object(NetworkDiscoveryService, "_get_rs_sg")
    @patch("montecarlodata.discovery.networking.is_overlap")
    def test_rs_network_with_peering(
        self,
        is_overlap_mock,
        sg_mock,
        intro_mock,
        cross_mock,
        req_mock,
        acc_mock,
        extra_mock,
        echo_mock,
        rs_mock,
    ):
        rs_cidr = "127.0.0.1/32"
        resource_props = SAMPLE_RESOURCE_PROPS
        resource_props.publicly_accessible = False

        sg_mock.return_value = ["foo"], True
        is_overlap_mock.return_value = False
        intro_mock.return_value = ["a"]
        cross_mock.return_value = ["b"]
        req_mock.return_value = ["c"]
        acc_mock.return_value = ["d"]
        extra_mock.return_value = ["e"]
        rs_mock.return_value = ["bar"]
        SAMPLE_DC_PROPS.resources_client.describe_vpc.return_value = Box(cidr_block=rs_cidr)
        SAMPLE_DC_PROPS.resources_client.describe_redshift_cluster.return_value = resource_props

        self._service.rs_network_recommender(
            dc_resource_props=SAMPLE_DC_PROPS, resource_identifier=SAMPLE_RESOURCE_ID
        )

        SAMPLE_DC_PROPS.resources_client.describe_redshift_cluster.assert_called_once_with(
            cluster_id=SAMPLE_RESOURCE_ID
        )
        SAMPLE_DC_PROPS.resources_client.describe_vpc.assert_called_once_with(
            SAMPLE_RESOURCE_PROPS.vpc_id
        )
        is_overlap_mock.assert_called_once_with(
            rs_cidr, SAMPLE_DC_PROPS.collector_props.parameters.vpc_cidr
        )
        sg_mock.assert_called_once_with(SAMPLE_DC_PROPS)

        echo_mock.assert_called_once_with(
            ["a", "b", "c", "d", "e"],
            header_col="The redshift cluster 'spensa' is not publicly accessible.",
        )
        intro_mock.assert_called_once_with(True)
        cross_mock.assert_called_once_with(SAMPLE_DC_PROPS, True)
        req_mock.assert_called_once_with(SAMPLE_DC_PROPS, True, rs_cidr)
        acc_mock.assert_called_once_with(SAMPLE_DC_PROPS, ["foo"], ["bar"])
        extra_mock.assert_called_once_with()

    @patch("montecarlodata.discovery.networking.complain_and_abort")
    def test_check_collector_validity(self, abort_mock):
        self.assertIsNone(self._service._check_collector_validity(SAMPLE_DC_PROPS))
        abort_mock.assert_not_called()

    @patch("montecarlodata.discovery.networking.complain_and_abort")
    def test_check_collector_validity_existing_vpc(self, abort_mock):
        props = copy.deepcopy(SAMPLE_DC_PROPS)
        props.collector_props.parameters.existing_vpc_id = "vpc-foo"

        self._service._check_collector_validity(props)
        abort_mock.assert_called_once_with(
            "Recommender does not support collectors deployed in customer managed VPCs."
        )

    def test_get_get_rs_sg_with_vpc(self):
        props = copy.deepcopy(SAMPLE_DC_PROPS)
        props.resource_props = SAMPLE_RESOURCE_PROPS

        sg, in_vpc = self._service._get_rs_sg(props)
        self.assertEqual(sg, ["sg-0987", "sg-6543"])
        self.assertTrue(in_vpc)

    @patch("montecarlodata.discovery.networking.complain_and_abort")
    def test_get_rs_sg_invalid(self, abort_mock):
        props = copy.deepcopy(SAMPLE_DC_PROPS)
        props.resource_props = SAMPLE_RESOURCE_PROPS
        props.resource_props.cluster_security_groups = ["foo"]

        self._service._get_rs_sg(props)
        abort_mock.assert_called_once_with(
            "Recommender does not support clusters with ambiguous hosting. "
            "Cluster cannot have security groups both inside and outside a VPC."
        )

    def test_get_rs_routes(self):
        props = copy.deepcopy(SAMPLE_DC_PROPS)
        props.resource_props = SAMPLE_RESOURCE_PROPS
        props.resource_props.cluster_subnet_group_name = "foo"

        props.resources_client.describe_cluster_subnet_groups.return_value = Box(
            subnets=[{"subnet_identifier": "bar"}]
        )
        props.resources_client.describe_routes.return_value = Box(
            route_tables=[{"route_table_id": "qux"}]
        )

        self.assertEqual(list(self._service._get_rs_routes(props)), ["qux"])
        props.resources_client.describe_routes.assert_called_once_with(filter_vals=["bar"])
        props.resources_client.describe_cluster_subnet_groups.assert_called_once_with(
            subnet_group="foo"
        )

    def test_get_ip_filtering_verbiage(self):
        sg = ["foo", "bar"]
        props = copy.deepcopy(SAMPLE_DC_PROPS)
        props.resource_props = SAMPLE_RESOURCE_PROPS

        self.assertEqual(
            self._service._get_ip_filtering_verbiage(props, sg),
            [
                "IP filtering is recommended. See steps below.",
                "",
                "Whitelist port '5439' for '127.0.0.1/32' in any of the following redshift "
                f"security groups - {sg}.",
                "https://docs.getmontecarlo.com/docs/network-connectivity#ip-filtering",
            ],
        )

    def test_get_rs_cross_account_verbiage(self):
        props = copy.deepcopy(SAMPLE_DC_PROPS)
        props.resource_props = SAMPLE_RESOURCE_PROPS
        self.assertEqual(
            self._service._get_rs_cross_account_verbiage(props, False),
            [
                "- Deploy the 'Create Peering Cross Account assumable role CloudFormation "
                "stack' with the following values -",
                "1. Data Collector AWS Account ID: 1234",
                "https://docs.getmontecarlo.com/docs/peering-templates#create-peering-cross-account-assumable-role-cloudformation-stack",
                "",
            ],
        )

    def test_get_rs_cross_account_verbiage_in_same_account(self):
        props = copy.deepcopy(SAMPLE_DC_PROPS)
        props.resource_props = SAMPLE_RESOURCE_PROPS
        self.assertEqual(self._service._get_rs_cross_account_verbiage(props, True), [])

    def test_get_rs_requester_verbiage(self):
        props = copy.deepcopy(SAMPLE_DC_PROPS)
        props.resource_props = SAMPLE_RESOURCE_PROPS
        props.resource_arn = AWSArn(SAMPLE_RESOURCE_PROPS.cluster_namespace_arn)

        self.assertEqual(
            self._service._get_rs_requester_verbiage(props, False, "127.0.0.1/32"),
            [
                "- Deploy the  'Create Requester CloudFormation stack' with the following values -",
                "1. Monte Carlo Data Collector VPC ID: 'vpc-1234'",
                "2. Monte Carlo Data Collector Security Group ID: 'sg-1234'",
                "3. Monte Carlo Data Collector Route Table ID: 'rtb-456'",
                "4. Warehouse/resource VPC ID: 'vpc-5678'",
                "5. Warehouse/resource AWS Account ID: '1234'",
                "6. Warehouse/resource AWS Region: 'us-east-1'",
                "7. Warehouse/resource CIDR Block: '127.0.0.1/32'",
                "8. VPC peer role for cross AWS account connections: <PeeringRole from the "
                "output of the previous stack>",
                "https://docs.getmontecarlo.com/docs/peering-templates#create-requester-cloudformation-stack",
                "",
            ],
        )

    def test_get_rs_accepter_verbiage(self):
        sg, routes = ["a"], ["b"]
        props = copy.deepcopy(SAMPLE_DC_PROPS)
        props.resource_props = SAMPLE_RESOURCE_PROPS
        props.resource_arn = AWSArn(SAMPLE_RESOURCE_PROPS.cluster_namespace_arn)

        self.assertEqual(
            self._service._get_rs_accepter_verbiage(props, sg, routes),
            [
                "- Deploy the  'Create Accepter CloudFormation stack' with the following values -",
                "1. Monte Carlo Data Collector CIDR Block: '10.30.0.0/16'",
                "2. Monte Carlo Data Collector Peering Connection: <PeeringConnection from "
                "the output of the previous stack>",
                "3. Resource / Warehouse Security Group: 'a'",
                "4. Resource / Warehouse Route Table #1: 'b'",
                "https://docs.getmontecarlo.com/docs/peering-templates#create-accepter-cloudformation-stack",
                "",
            ],
        )

    def test_get_overlap_verbiage(self):
        self.assertEqual(
            self._service._get_overlap_verbiage("a", "b"),
            "Collector range (a) overlaps with resource (b). Peering is not possible when peered VPCs use overlapping "
            "CIDR blocks. Please redeploy the collector with a custom CIDR block then rerun recommender.",
        )

    def test_get_peering_intro_verbiage(self):
        self.assertEqual(
            self._service._get_peering_intro_verbiage(True),
            [
                "VPC Peering is recommended. See the outlined steps for each of the 2 "
                "sections below.",
                "",
                "The CloudFormation template can be found by following the link at the end of "
                "each section.",
                "Please complete all 2 sections, and for any additional help, reach out to "
                "your Monte Carlo representative.",
                "",
            ],
        )

    def test_get_extra_verbiage(self):
        self.assertEqual(
            self._service._get_extra_verbiage(),
            [
                "If the cluster's subnets use a non-default ACL the collector "
                "CIDR Block also likely need to be whitelisted."
            ],
        )

    def test_get_friendly_sg_verbiage(self):
        self.assertEqual(self._service._get_friendly_sg_verbiage(["a"]), "'a'")

    def test_get_friendly_sg_verbiage_multi(self):
        self.assertEqual(
            self._service._get_friendly_sg_verbiage(["a", "b"]),
            "Any of the following - a,b",
        )

    @patch("montecarlodata.discovery.networking.click")
    def test_echo_recommendation(self, click_mock):
        self._service._echo_recommendation(["a", "b"], table_format="plain")
        click_mock.echo.assert_called_once_with("Recommendations\na\nb")



import os
import urllib.request
from typing import Optional
from urllib.parse import urlparse

import click
import requests
from tabulate import tabulate

from montecarlodata.config import Config
from montecarlodata.data_exports.catalog import DATA_EXPORT_CATALOG
from montecarlodata.data_exports.fields import (
    EXPECTED_GET_DATA_EXPORT_URL,
    FILE_SCHEME,
    LIST_DATA_EXPORTS_HEADERS,
    S3_SCHEME,
    SCHEME_DELIM,
)
from montecarlodata.errors import complain_and_abort, manage_errors
from montecarlodata.fs_utils import mkdirs
from montecarlodata.queries.data_exports import GET_DATA_EXPORT_URL
from montecarlodata.utils import AwsClientWrapper, GqlWrapper


class DataExportService:
    def __init__(
        self,
        config: Config,
        command_name: str,
        request_wrapper: Optional[GqlWrapper] = None,
    ):
        self._abort_on_error = True
        self._command_name = command_name

        self._request_wrapper = request_wrapper or GqlWrapper(
            config,
            command_name=self._command_name,
        )

        self.scheme_handler = {
            FILE_SCHEME: self._save_data_export_to_disk,
            S3_SCHEME: self._save_data_export_to_s3,
        }

    @manage_errors
    def echo_data_exports(
        self,
        headers: str = "firstrow",
        table_format: str = "fancy_grid",
    ) -> None:
        """
        Echo Data Exports as a pretty table
        """
        table = [LIST_DATA_EXPORTS_HEADERS]
        for data_export in DATA_EXPORT_CATALOG:
            table.append(
                [
                    f"{data_export.title} ({data_export.name})",
                    (
                        data_export.description
                        if data_export.description.endswith(".")
                        else f"{data_export.description}."
                    ),
                ]
            )
        click.echo(tabulate(table, headers=headers, tablefmt=table_format, maxcolwidths=100))

    @manage_errors
    def get_data_export(
        self,
        data_export: str,
        destination: str,
        aws_profile: Optional[str] = None,
        dry: bool = False,
    ) -> None:
        """
        Get Data Export, if available, and either persist to S3 or local file system
        """
        parsed_destination = urlparse(destination)
        scheme = parsed_destination.scheme
        if scheme == S3_SCHEME and not aws_profile:
            raise ValueError("Cannot use an s3 destination without specifying an AWS profile.")

        netloc_with_path = parsed_destination.geturl().replace(f"{scheme}{SCHEME_DELIM}", "", 1)

        try:
            handler = self.scheme_handler[scheme]
        except KeyError:
            complain_and_abort("Scheme either missing or not supported.")
        else:
            data_export_url = self._get_data_export_url(data_export=data_export)
            if dry:
                click.echo(data_export_url)
                return
            click.echo(f"Saving data export to '{destination}'.")
            handler(
                data_export_url=data_export_url,
                destination=netloc_with_path,
                aws_profile=aws_profile,
            )
            click.echo("Complete. Have a nice day!")

    def _get_data_export_url(self, data_export: str) -> str:
        """
        Get Data Export URL from the monolith
        """
        resp = self._request_wrapper.make_request_v2(
            query=GET_DATA_EXPORT_URL,
            operation=EXPECTED_GET_DATA_EXPORT_URL,
            service="data_exports_service",
            variables=dict(
                data_export_name=data_export,
            ),
        )
        url = resp.data.url  # type: ignore
        if not url:
            complain_and_abort(
                "Data Export not found. This Data Export might not be available for your account."
            )
        return url

    def _save_data_export_to_disk(self, data_export_url: str, destination: str, **kwargs) -> None:
        """
        Save Data Export to the local filesystem
        """
        if not destination:
            complain_and_abort(
                f"Invalid path. Expected format: '{FILE_SCHEME}{SCHEME_DELIM}folder/file.csv'"
            )
        mkdirs(os.path.dirname(destination))
        urllib.request.urlretrieve(data_export_url, destination)

    def _save_data_export_to_s3(
        self, data_export_url: str, destination: str, aws_profile: Optional[str] = None
    ) -> None:
        """
        Save Data Export to S3
        """
        try:
            bucket, key = destination.split("/", maxsplit=1)
            if not key:
                raise ValueError
        except (ValueError, AttributeError):
            complain_and_abort(
                f"Invalid path. Expected format: '{S3_SCHEME}{SCHEME_DELIM}bucket/key.csv'"
            )
        else:
            AwsClientWrapper(profile_name=aws_profile).upload_stream_to_s3(
                data=requests.get(data_export_url, stream=True).raw, bucket=bucket, key=key
            )



# Available platform types
AWS = "AWS"
AZURE = "AZURE"
GCP = "GCP"
AWS_GENERIC = "AWS_GENERIC"

# Available agent types
DATA_STORE_AGENT = "DATA_STORE_AGENT"
REMOTE_AGENT = "REMOTE_AGENT"

# Available authentication types
AZURE_STORAGE_ACCOUNT_KEYS = "AZURE_STORAGE_ACCOUNT_KEYS"
GCP_JSON_SERVICE_ACCOUNT_KEY = "GCP_JSON_SERVICE_ACCOUNT_KEY"
GCP_WORKLOAD_IDENTITY_FEDERATION = "GCP_WORKLOAD_IDENTITY_FEDERATION"
AWS_VPCE = "AWS_VPCE"
AWS_ASSUMABLE_ROLE = "AWS_ASSUMABLE_ROLE"
AZURE_FUNCTION_APP_KEY = "AZURE_FUNCTION_APP_KEY"
OAUTH2_CLIENT_CREDENTIALS = "OAUTH2_CLIENT_CREDENTIALS"
OAUTH2_PASSWORD = "OAUTH2_PASSWORD"

# Available storage types
AZURE_BLOB = "AZURE_BLOB"
GCS = "GCS"
S3 = "S3"

# Available wrapper types
TERRAFORM = "TERRAFORM"



# Queries related to Data Exports

GET_DATA_EXPORT_URL = """
    query getDataExportUrl($dataExportName: DataExportNames!) {
        getDataExportUrl(dataExportName: $dataExportName) {
            url
        }
    }
"""



"""
Command line tools
"""

import copy
import json
from datetime import datetime
from typing import Optional, Set, cast

import click
from click import Context, Option

import montecarlodata.settings as settings
from montecarlodata.common.common import normalize_gql


class AdvancedOptions(Option):
    """
    Set mutually exclusive options, groups of required options and hidden prompts.
    Errors out on conflict or any missing values and appends text to help.

    Usage example -
        click.option('--foo' cls=MutuallyExclusiveOptions, mutually_exclusive_options=['bar']),
        click.option('--bar', cls=MutuallyExclusiveOptions, mutually_exclusive_options=['foo'])
    """

    def __init__(self, *args, **kwargs):
        help_text = kwargs.get("help", "")
        self._friendly_mutual_args = self._friendly_required_args = (
            self._friendly_at_least_one_set
        ) = None

        self.mutually_exclusive_options = set(kwargs.pop("mutually_exclusive_options", []))
        self.required_with_options = set(kwargs.pop("required_with_options", []))
        self.at_least_one_set = set(kwargs.pop("at_least_one_set", []))
        self.prompt_if_requested = kwargs.pop("prompt_if_requested", False)

        self.values_with_required_options = set(kwargs.pop("values_with_required_options", []))
        self.required_options_for_values = set(kwargs.pop("required_options_for_values", []))

        # Update help for options and create friendly args (i.e. human readable).
        if self.mutually_exclusive_options:
            self._friendly_mutual_args = self._create_friendly_args(
                options=self.mutually_exclusive_options
            )
            help_text = f"{help_text} This option cannot be used with {self._friendly_mutual_args}."
        if self.required_with_options:
            self._friendly_required_args = self._create_friendly_args(
                options=self.required_with_options
            )
            help_text = f"{help_text} This option requires setting {self._friendly_required_args}."
        if self.required_options_for_values:
            self._friendly_required_options_for_values = self._create_friendly_args(
                self.required_options_for_values
            )
            help_text = (
                f"{help_text} This option requires setting "
                f"{self._friendly_required_options_for_values} "
                f"when it is set to one of these values: {self.values_with_required_options}."
            )
        self._friendly_at_least_one_set = self._create_friendly_args(self.at_least_one_set)
        kwargs["help"] = help_text
        super(AdvancedOptions, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        """
        Handle any conflicts, missing values or any required hidden prompts.
        """
        cli_friendly_name = normalize_gql(self.name)  # type: ignore

        if self.name in opts:
            name_opt = cast(str, opts[self.name])  # type: ignore  (should always be set)
            if self.mutually_exclusive_options.intersection(opts):
                raise click.BadParameter(
                    f"Cannot use '{cli_friendly_name}' with {self._friendly_mutual_args}."
                )
            if not self.required_with_options.issubset(opts):
                raise click.BadParameter(
                    f"Cannot use '{cli_friendly_name}' without {self._friendly_required_args}."
                )
            if (
                name_opt in self.values_with_required_options
                and not self.required_options_for_values.issubset(opts)
            ):
                raise click.BadParameter(
                    f"Cannot use '{cli_friendly_name}' with value "
                    f"'{name_opt}' without "
                    f"{self._friendly_required_options_for_values}."
                )
            if self.prompt_if_requested and name_opt == settings.SHOW_PROMPT_VALUE:
                opts[self.name] = click.prompt(  # type: ignore
                    cli_friendly_name,  # type: ignore
                    hide_input=True,
                )
            else:
                self.prompt = None

        if self.at_least_one_set and not any(x in opts for x in self.at_least_one_set):
            raise click.BadParameter(
                f"Missing required options from {self._friendly_at_least_one_set}."
            )

        return super(AdvancedOptions, self).handle_parse_result(ctx, opts, args)

    @staticmethod
    def _create_friendly_args(options: Set[str]) -> str:
        """
        Build a more human readable sentence from options.
        """
        friendly_options = list(copy.deepcopy(options))
        last_element = friendly_options.pop() if len(friendly_options) > 1 else None

        friendly_options = ", ".join([f"'{normalize_gql(option)}'" for option in friendly_options])
        if last_element:
            friendly_options = f"{friendly_options}, and '{last_element}'"
        return friendly_options


def add_common_options(options):
    """
    Convenience decorator for shared options (i.e. options that are common across commands)
    """

    def _add_common_options(function):
        for option in options[::-1]:
            function = option(function)
        return function

    return _add_common_options


def validate_json_callback(ctx_, param_, value):
    """
    Convenience callback to help validate (and load) JSON in option strings
    """
    try:
        return json.loads(value)
    except json.decoder.JSONDecodeError as err:
        raise click.BadParameter(f"Malformed JSON - {err}")
    except TypeError:
        return value


def convert_uuid_callback(ctx_, param_, value):
    """
    Convenience callback to convert UUIDs into strings
    """
    if value:
        return str(value)  # str(None) returns 'None'
    return value


def convert_empty_str_callback(ctx_, param_, value):
    """
    Convenience callback to convert an empty (or empty like) string into None
    """
    if bool(value and not value.isspace()):
        return value


def dump_help(cmd, parent: Optional[Context] = None):
    """
    Recursively echos help text for a cmd
    """
    ctx = click.core.Context(cmd, info_name=cmd.name, parent=parent)
    click.echo(cmd.get_help(ctx), err=True)
    for sub in getattr(cmd, "commands", {}).values():
        dump_help(sub, ctx)


def format_date(dt: str) -> str:
    """Format string from ISO format to YYYY-MM-DD"""
    return datetime.fromisoformat(dt).strftime("%Y-%m-%d")


def format_datetime(dt: str) -> str:
    """Format string from ISO format to YYYY-MM-DD HH:MM:SS"""
    return datetime.fromisoformat(dt).strftime("%Y-%m-%d %H:%M:%S")



from unittest import TestCase
from unittest.mock import Mock, call, patch

from box import Box
from parameterized import parameterized
from pycarlo.core import Client, Mutation, Query

from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.data_lake.databricks import (
    DEFAULT_GATEWAY_URL,
    DEFAULT_SECRET_NAME,
    DEFAULT_SECRET_SCOPE,
    DatabricksOnboardingService,
)
from montecarlodata.integrations.onboarding.fields import (
    DATA_LAKE_WAREHOUSE_TYPE,
    DATABRICKS_DELTA_CONNECTION_TYPE,
    DATABRICKS_METASTORE_CONNECTION_TYPE,
    DATABRICKS_METASTORE_SQL_WAREHOUSE_CONNECTION_TYPE,
)
from montecarlodata.queries.onboarding import (
    TEST_DATABRICKS_CRED_MUTATION,
    DatabricksSqlWarehouseOnboardingQueries,
)
from montecarlodata.utils import AwsClientWrapper
from tests.test_base_onboarding import _SAMPLE_BASE_OPTIONS
from tests.test_common_user import _SAMPLE_CONFIG


class DatabricksMetastoreOnboardingTest(TestCase):
    def setUp(self) -> None:
        self._mc_client_mock = Mock(autospec=Client)
        self._user_service_mock = Mock(autospec=UserService)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = DatabricksOnboardingService(
            _SAMPLE_CONFIG,
            command_name="test",
            mc_client=self._mc_client_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock,
        )

    @parameterized.expand(
        [
            (DATABRICKS_METASTORE_CONNECTION_TYPE, [], {}, 3),
            (DATABRICKS_DELTA_CONNECTION_TYPE, ["hive"], {"project_id": "hive"}, 4),
            (
                DATABRICKS_DELTA_CONNECTION_TYPE,
                ["awsdatacatalog"],
                {"project_id": "awsdatacatalog"},
                4,
            ),
        ]
    )
    @patch.object(DatabricksOnboardingService, "onboard")
    def test_databricks_flow(
        self,
        connection_type,
        projects,
        expected_projects,
        expected_call_count,
        onboard_mock,
    ):
        options = {
            "databricks_workspace_url": "databricks_workspace_url",
            "databricks_workspace_id": "databricks_workspace_id",
            "databricks_cluster_id": "databricks_cluster_id",
            "databricks_token": "databricks_token",
            "databricks_secret_key": DEFAULT_SECRET_NAME,
            "databricks_secret_scope": DEFAULT_SECRET_SCOPE,
        }
        job_info = dict(
            databricks_job_id="databricks_job_id",
            databricks_job_name="databricks_job_name",
            databricks_notebook_path="databricks_notebook_path",
            databricks_notebook_source="databricks_notebook_path",
            databricks_notebook_version="123",
        )

        self._mc_client_mock.side_effect = [
            Mock(),
            Mock(),
            Box(
                dict(
                    create_databricks_notebook_job=dict(
                        databricks=dict(
                            workspace_job_id=job_info["databricks_job_id"],
                            workspace_job_name=job_info["databricks_job_name"],
                            workspace_notebook_path=job_info["databricks_notebook_path"],
                            notebook_source=job_info["databricks_notebook_source"],
                            notebook_version=job_info["databricks_notebook_version"],
                        ),
                    )
                )
            ),
            Box(dict(get_projects=dict(projects=projects))),
        ]

        self._service.onboard_databricks_metastore(
            connection_type=connection_type, **options, **_SAMPLE_BASE_OPTIONS
        )

        expected_options = {**options, **_SAMPLE_BASE_OPTIONS}
        expected_job_limits = {
            **job_info,
            "integration_gateway": {
                "gateway_url": DEFAULT_GATEWAY_URL,
                "databricks_secret_key": DEFAULT_SECRET_NAME,
                "databricks_secret_scope": DEFAULT_SECRET_SCOPE,
            },
            **expected_projects,
        }

        onboard_mock.assert_called_once_with(
            validation_query=TEST_DATABRICKS_CRED_MUTATION,
            validation_response="testDatabricksCredentials",
            connection_type=connection_type,
            job_limits=expected_job_limits,
            **expected_options,
        )
        self.assertEqual(self._mc_client_mock.call_count, expected_call_count)

    @patch.object(DatabricksOnboardingService, "onboard")
    def test_databricks_gql_api_calls(self, onboard_mock):
        options = {
            "databricks_workspace_url": "databricks_workspace_url",
            "databricks_workspace_id": "databricks_workspace_id",
            "databricks_cluster_id": "databricks_cluster_id",
            "databricks_token": "databricks_token",
            "databricks_secret_key": DEFAULT_SECRET_NAME,
            "databricks_secret_scope": DEFAULT_SECRET_SCOPE,
            "dc_id": "data collector id",
        }
        job_info = dict(
            databricks_job_id="databricks_job_id",
            databricks_job_name="databricks_job_name",
            databricks_notebook_path="databricks_notebook_path",
            databricks_notebook_source="databricks_notebook_path",
            databricks_notebook_version="123",
        )

        databricks_config = {
            "databricks_workspace_url": "databricks_workspace_url",
            "databricks_workspace_id": "databricks_workspace_id",
            "databricks_cluster_id": "databricks_cluster_id",
            "databricks_token": "databricks_token",
        }

        self._mc_client_mock.side_effect = [
            Mock(),
            Mock(),
            Box(
                dict(
                    create_databricks_notebook_job=dict(
                        databricks=dict(
                            workspace_job_id=job_info["databricks_job_id"],
                            workspace_job_name=job_info["databricks_job_name"],
                            workspace_notebook_path=job_info["databricks_notebook_path"],
                            notebook_source=job_info["databricks_notebook_source"],
                            notebook_version=job_info["databricks_notebook_version"],
                        )
                    )
                )
            ),
            Box(dict(get_projects=dict(projects=[]))),
        ]

        self._service.onboard_databricks_metastore(
            connection_type=DATABRICKS_METASTORE_CONNECTION_TYPE,
            **options,
            **_SAMPLE_BASE_OPTIONS,
        )

        expected_validation_call = Query()
        expected_validation_call.validate_connection_type(
            warehouse_type=DATA_LAKE_WAREHOUSE_TYPE,
            connection_type=DATABRICKS_METASTORE_CONNECTION_TYPE,
        )

        expected_secret_call = Mutation()
        expected_secret_call.create_databricks_secret(
            databricks_config=databricks_config,
            secret_name=DEFAULT_SECRET_NAME,
            scope_name=DEFAULT_SECRET_SCOPE,
            connection_options={"dc_id": "data collector id"},
        )

        expected_notebook_job_call = Mutation()
        expected_notebook_job_call.create_databricks_notebook_job(
            databricks_config=databricks_config,
            connection_options={"dc_id": "data collector id"},
        )

        expected_calls = [
            call(
                expected_validation_call,
                additional_headers={
                    "x-mcd-telemetry-reason": "cli",
                    "x-mcd-telemetry-service": "databricks_onboarding_service",
                    "x-mcd-telemetry-command": "test",
                },
            ),
            call(
                expected_secret_call,
                additional_headers={
                    "x-mcd-telemetry-reason": "cli",
                    "x-mcd-telemetry-service": "databricks_onboarding_service",
                    "x-mcd-telemetry-command": "test",
                },
            ),
            call(
                expected_notebook_job_call,
                additional_headers={
                    "x-mcd-telemetry-reason": "cli",
                    "x-mcd-telemetry-service": "databricks_onboarding_service",
                    "x-mcd-telemetry-command": "test",
                },
            ),
        ]
        self.assertEqual(len(self._mc_client_mock.call_args_list), len(expected_calls))

        for i, expected_call in enumerate(expected_calls):
            # This is the same way that the SDK tests that two calls are the same
            # https://github.com/monte-carlo-data/python-sdk/blob/62f1ab7a8404119da4621b224e77bbbdd5dc938c/tests/test_operations.py#L21
            self.assertEqual(
                str(self._mc_client_mock.call_args_list[i]).strip(),
                str(expected_call).strip(),
            )

    def test_update_databricks_notebook_gql_calls(self):
        options = {"connection_id": "123456789"}
        self._service.update_databricks_notebook(**options)
        expected_mutation = Mutation()
        expected_mutation.update_databricks_notebook(**options)
        expected_call = call(
            expected_mutation,
            additional_headers={
                "x-mcd-telemetry-reason": "cli",
                "x-mcd-telemetry-service": "databricks_onboarding_service",
                "x-mcd-telemetry-command": "test",
            },
        )
        self.assertEqual(
            str(self._mc_client_mock.call_args_list[0]).strip(),
            str(expected_call).strip(),
        )

    def test_get_databricks_job_info_gql_calls(self):
        options = {"connection_id": "123456789"}
        self._mc_client_mock.return_value = Mock(
            get_databricks_metadata_job_info=[
                Mock(
                    workspace_job_id="123456",
                    workspace_job_name="test_job_name",
                    workspace_notebook_path="/some/path",
                    notebook_source="test_source",
                    notebook_version="123",
                )
            ]
        )
        self._service.get_databricks_job_info(**options)
        expected_query = Query()
        expected_query.get_databricks_metadata_job_info(**options)
        expected_call = call(
            expected_query,
            additional_headers={
                "x-mcd-telemetry-reason": "cli",
                "x-mcd-telemetry-service": "databricks_onboarding_service",
                "x-mcd-telemetry-command": "test",
            },
        )
        self.assertEqual(
            str(self._mc_client_mock.call_args_list[0]).strip(),
            str(expected_call).strip(),
        )

    def test_get_current_databricks_notebook_version_gql_calls(self):
        self._service.get_current_databricks_notebook_version()
        expected_query = Query()
        expected_query.get_current_databricks_notebook_version()
        expected_call = call(
            expected_query,
            additional_headers={
                "x-mcd-telemetry-reason": "cli",
                "x-mcd-telemetry-service": "databricks_onboarding_service",
                "x-mcd-telemetry-command": "test",
            },
        )
        self.assertEqual(
            str(self._mc_client_mock.call_args_list[0]).strip(),
            str(expected_call).strip(),
        )

    @patch.object(DatabricksOnboardingService, "onboard")
    def test_onboard_databricks_metastore_sql_warehouse(
        self,
        onboard_mock,
    ):
        options = {
            "databricks_workspace_url": "databricks_workspace_url",
            "databricks_warehouse_id": "databricks_warehouse_id",
            "databricks_workspace_id": "databricks_workspace_id",
            "databricks_token": "databricks_token",
        }

        self._service.onboard_databricks_sql_warehouse(
            connection_type=DATABRICKS_METASTORE_SQL_WAREHOUSE_CONNECTION_TYPE,
            **options,
        )

        onboard_mock.assert_called_once_with(
            validation_query=DatabricksSqlWarehouseOnboardingQueries.test_credentials.query,
            validation_response="testDatabricksSqlWarehouseCredentials",
            connection_type=DATABRICKS_METASTORE_SQL_WAREHOUSE_CONNECTION_TYPE,
            **options,
        )



import os
import pathlib
from typing import Callable, Dict
from unittest import TestCase
from unittest.mock import Mock

import click
from box import Box

from montecarlodata.common.common import read_as_json_string
from montecarlodata.common.data import MonolithResponse
from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.data_lake.events import (
    EventsOnboardingService,
)
from montecarlodata.integrations.onboarding.fields import (
    EXPECTED_CONFIGURE_METADATA_EVENTS_GQL_RESPONSE_FIELD,
    EXPECTED_CONFIGURE_QUERY_LOG_EVENTS_GQL_RESPONSE_FIELD,
    EXPECTED_DISABLE_METADATA_EVENTS_GQL_RESPONSE_FIELD,
    EXPECTED_DISABLE_QUERY_LOG_EVENTS_GQL_RESPONSE_FIELD,
)
from montecarlodata.queries.onboarding import (
    CONFIGURE_METADATA_EVENTS_MUTATION,
    CONFIGURE_QUERY_LOG_EVENTS_MUTATION,
    DISABLE_METADATA_EVENTS_MUTATION,
    DISABLE_QUERY_LOG_EVENTS_MUTATION,
)
from montecarlodata.utils import AwsClientWrapper, GqlWrapper
from tests.test_common_user import _SAMPLE_CONFIG


class EventsOnboardingTest(TestCase):
    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = EventsOnboardingService(
            _SAMPLE_CONFIG,
            command_name="test",
            request_wrapper=self._request_wrapper_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock,
        )

    def test_configure_metadata_events(self):
        kwargs = {"connection_type": "glue", "name": "warehouse"}

        self._test(
            func=self._service.configure_metadata_events,
            kwargs=kwargs,
            expected_mutation=CONFIGURE_METADATA_EVENTS_MUTATION,
            expected_operation=EXPECTED_CONFIGURE_METADATA_EVENTS_GQL_RESPONSE_FIELD,
            expected_variables=kwargs,
        )

    def test_configure_query_log_events(self):
        mapping_file = os.path.join(pathlib.Path(__file__).parent.resolve(), "sample_mapping.json")
        kwargs = {
            "connection_type": "hive-s3",
            "format_type": "custom",
            "location": "foo",
            "source_format": "bar",
            "mapping_file": mapping_file,
            "name": "warehouse",
        }

        expected_variables = {**kwargs, "mapping": read_as_json_string(mapping_file)}
        expected_variables.pop("mapping_file")

        self._test(
            func=self._service.configure_query_log_events,
            kwargs=kwargs,
            expected_mutation=CONFIGURE_QUERY_LOG_EVENTS_MUTATION,
            expected_operation=EXPECTED_CONFIGURE_QUERY_LOG_EVENTS_GQL_RESPONSE_FIELD,
            expected_variables=expected_variables,
        )

    def test_disable_metadata_events(self):
        kwargs = {"name": "warehouse"}

        self._test(
            func=self._service.disable_metadata_events,
            kwargs=kwargs,
            expected_mutation=DISABLE_METADATA_EVENTS_MUTATION,
            expected_operation=EXPECTED_DISABLE_METADATA_EVENTS_GQL_RESPONSE_FIELD,
            expected_variables=kwargs,
        )

    def test_disable_query_log_events(self):
        kwargs = {"name": "warehouse"}

        self._test(
            func=self._service.disable_query_log_events,
            kwargs=kwargs,
            expected_mutation=DISABLE_QUERY_LOG_EVENTS_MUTATION,
            expected_operation=EXPECTED_DISABLE_QUERY_LOG_EVENTS_GQL_RESPONSE_FIELD,
            expected_variables=kwargs,
        )

    def _test(
        self,
        func: Callable,
        kwargs: Dict,
        expected_mutation: str,
        expected_operation: str,
        expected_variables: Dict,
    ):
        # test a successful GraphQL response
        self._verify_graphql_request(
            func=func,
            kwargs=kwargs,
            expected_mutation=expected_mutation,
            expected_operation=expected_operation,
            expected_variables=expected_variables,
            success=True,
        )

        # test a failed GraphQL response
        self._verify_graphql_request(
            func=func,
            kwargs=kwargs,
            expected_mutation=expected_mutation,
            expected_operation=expected_operation,
            expected_variables=expected_variables,
            success=False,
        )

    def _verify_graphql_request(
        self,
        func: Callable,
        kwargs: Dict,
        success: bool,
        expected_mutation: str,
        expected_operation: str,
        expected_variables: Dict,
    ):
        # given
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(
            data=Box({"success": success})
        )

        # when
        if success:
            func(**kwargs)
        else:
            self.assertRaises(click.exceptions.Abort, func, **kwargs)

        # then
        self._request_wrapper_mock.make_request_v2.assert_called_with(
            query=expected_mutation,
            operation=expected_operation,
            service="events_onboarding_service",
            variables=expected_variables,
        )



from typing import Optional

import click

from montecarlodata.integrations.keys import IntegrationKeyService
from montecarlodata.keys.airflow import AirflowService

_AIRFLOW_CALLBACKS_SCOPE = "AirflowCallbacks"


@click.group(help="Manage Integration Gateway Keys")
def keys():
    pass


@keys.command(name="add-airflow", help="Add a new Airflow Integration Key")
@click.pass_obj
@click.option(
    "--name",
    required=False,
    help="Name of the Airflow Connection, required to disambiguate "
    "if multiple Airflow connections are defined",
)
@click.option("--description", required=True, help="Description for the key")
def add_airflow_integration_key(ctx, name: Optional[str] = None, description: Optional[str] = None):
    config = ctx["config"]
    resource_uuid = AirflowService(
        config=config,
        command_name="keys add_airflow_integration_key",
    ).get_airflow_resource_uuid(name=name)
    IntegrationKeyService(
        config=config,
        command_name="keys add_airflow_integration_key",
    ).create(
        description=description,
        scope=_AIRFLOW_CALLBACKS_SCOPE,
        warehouse_ids=[resource_uuid],
    )


@keys.command(name="list-airflow", help="List existing Airflow Integration Keys in the account")
@click.pass_obj
@click.option(
    "--name",
    required=False,
    help="Show keys only for the Airflow Connection with this name",
)
def list_airflow_integration_keys(ctx, name: Optional[str] = None):
    resource_uuid = (
        AirflowService(
            config=ctx["config"],
            command_name="keys list_airflow_integration_keys",
        ).get_airflow_resource_uuid(name=name)
        if name
        else None
    )
    IntegrationKeyService(
        config=ctx["config"],
        command_name="keys list_airflow_integration_keys",
    ).get_all(scope=_AIRFLOW_CALLBACKS_SCOPE, resource_uuid=resource_uuid)


@keys.command(name="delete-airflow", help="Delete an existing Airflow Integration Key")
@click.pass_obj
@click.option("--key-id", required=True, help="Id of the key to delete")
def delete_airflow_integration_key(ctx, key_id: str):
    IntegrationKeyService(
        config=ctx["config"],
        command_name="keys delete_airflow_integration_key",
    ).delete(key_id=key_id)



# Query responses
EXPECTED_GET_INSIGHTS_FIELD = "getInsights"
EXPECTED_GET_REPORT_FIELD = "getReportUrl"

# Insight Names
KEY_ASSETS_INSIGHT_NAME = "key_assets"
COVERAGE_OVERVIEW_INSIGHT_NAME = "monitors"
CLEANUP_SUGGESTIONS_INSIGHT_NAME = "cleanup_suggestions"
EVENTS_INSIGHT_NAME = "events"
READ_WRITE_INSIGHT_NAME = "table_read_write_stats"
INCIDENT_QUERY_INSIGHT_NAME = "incident_query_changes"
DETERIORATING_QUERY_INSIGHT_NAME = "query_runtime_trends"
RULES_AND_SLIS_INSIGHT_NAME = "rule_and_sli_results"

# Supported Schemes
FILE_SCHEME = "file"
S3_SCHEME = "s3"
SCHEME_DELIM = "://"

# Insights extension
INSIGHTS_DEFAULT_EXTENSION = ".csv"

# Insights headers
LIST_INSIGHTS_HEADERS = ["Insight (Name)", "Description", "Available"]



MATCH_AND_CREATE_BI_WAREHOUSE_SOURCES = """
mutation matchAndCreateBiWarehouseSources($biContainerId: UUID!, $biWarehouseSources: [BiWarehouseSourcesInput]) {
 matchAndCreateBiWarehouseSources(biContainerId: $biContainerId, biWarehouseSources: $biWarehouseSources) {
        matchingBiWarehouseSources{
    matchSuccessful
    biWarehouseSources {
      warehouseResourceId
      warehouseResourceType
      biWarehouseId
    }
    rawBiWarehouseConnections
    rawWarehouseConnections
  }
 }
}"""



from montecarlodata.common.common import read_as_base64
from montecarlodata.config import Config
from montecarlodata.errors import manage_errors
from montecarlodata.integrations.onboarding.base import BaseOnboardingService
from montecarlodata.integrations.onboarding.fields import (
    EXPECTED_ADD_BI_RESPONSE_FIELD,
    EXPECTED_LOOKER_GIT_CLONE_RESPONSE_FIELD,
    EXPECTED_LOOKER_METADATA_RESPONSE_FIELD,
    EXPECTED_POWER_BI_RESPONSE_FIELD,
    EXPECTED_TEST_TABLEAU_RESPONSE_FIELD,
    LOOKER_BI_TYPE,
    LOOKER_GIT_CLONE_CONNECTION_TYPE,
    LOOKER_MD_CONNECTION_TYPE,
    POWER_BI_BI_TYPE,
    POWER_BI_CONNECTION_TYPE,
    TABLEAU_BI_TYPE,
    TABLEAU_CONNECTION_TYPE,
)
from montecarlodata.queries.onboarding import (
    ADD_BI_CONNECTION_MUTATION,
    TEST_LOOKER_GIT_CLONE_CRED_MUTATION,
    TEST_LOOKER_METADATA_CRED_MUTATION,
    TEST_POWER_BI_CRED_MUTATION,
    TEST_TABLEAU_CRED_MUTATION,
)


class ReportsOnboardingService(BaseOnboardingService):
    def __init__(self, config: Config, command_name: str, **kwargs):
        super().__init__(config, command_name=command_name, **kwargs)

    @manage_errors
    def onboard_tableau(self, **kwargs) -> None:
        """
        Onboard a tableau connection
        """
        self.onboard(
            validation_query=TEST_TABLEAU_CRED_MUTATION,
            validation_response=EXPECTED_TEST_TABLEAU_RESPONSE_FIELD,
            connection_query=ADD_BI_CONNECTION_MUTATION,
            connection_response=EXPECTED_ADD_BI_RESPONSE_FIELD,
            connection_type=TABLEAU_CONNECTION_TYPE,
            warehouse_type=TABLEAU_BI_TYPE,
            **kwargs,
        )

    @manage_errors
    def onboard_looker_metadata(self, **kwargs) -> None:
        """
        Onboard a looker metadata connection
        """
        self.onboard(
            validation_query=TEST_LOOKER_METADATA_CRED_MUTATION,
            validation_response=EXPECTED_LOOKER_METADATA_RESPONSE_FIELD,
            connection_query=ADD_BI_CONNECTION_MUTATION,
            connection_response=EXPECTED_ADD_BI_RESPONSE_FIELD,
            connection_type=LOOKER_MD_CONNECTION_TYPE,
            warehouse_type=LOOKER_BI_TYPE,
            **kwargs,
        )

    @manage_errors
    def onboard_looker_git(self, **kwargs) -> None:
        """
        Onboard a looker git ssh connection
        """
        if kwargs.get("ssh_key"):
            kwargs["ssh_key"] = read_as_base64(kwargs.pop("ssh_key")).decode("utf-8")
        self.onboard(
            validation_query=TEST_LOOKER_GIT_CLONE_CRED_MUTATION,
            validation_response=EXPECTED_LOOKER_GIT_CLONE_RESPONSE_FIELD,
            connection_query=ADD_BI_CONNECTION_MUTATION,
            connection_response=EXPECTED_ADD_BI_RESPONSE_FIELD,
            connection_type=LOOKER_GIT_CLONE_CONNECTION_TYPE,
            warehouse_type=LOOKER_BI_TYPE,
            **kwargs,
        )

    @manage_errors
    def onboard_power_bi(self, **kwargs) -> None:
        """
        Onboard a Power BI connection
        """
        self.onboard(
            validation_query=TEST_POWER_BI_CRED_MUTATION,
            validation_response=EXPECTED_POWER_BI_RESPONSE_FIELD,
            connection_query=ADD_BI_CONNECTION_MUTATION,
            connection_response=EXPECTED_ADD_BI_RESPONSE_FIELD,
            connection_type=POWER_BI_CONNECTION_TYPE,
            warehouse_type=POWER_BI_BI_TYPE,
            **kwargs,
        )
